; Constant registers 
; c0-c3 - world space transposed
; c8-c11 - world * view * proj
; c12 - Light Direction (In World Space)
; c24 - camera direction
; c33 - four constants with 0.5f
;
; Input registers
; v0 - Position
; v3 - Normal
; v7 - Texture 
; v8 - Tangent
vs.1.1

; position in clip space
m4x4 oPos, v0, c8

m3x3 r3, v8, c0	; generate tangent U
m3x3 r5, v3, c0	; generate normal W

; Cross product
; generate binormal V
mul r0, -r3.zxyw, r5.yzxw;
mad r4, -r3.yzxw, r5.zxyw,-r0;

; position in world space
m4x4 r2, v0, c0			

; get a vector toward the camera/eye V
add r2, -r2, c24

; normalize eye vector V
dp3 r11.x,  r2.xyz, r2.xyz	
rsq r11.xyz, r11.x			
mul r2.xyz, r2.xyz, r11.xyz	

add r2.xyz, r2.xyz, -c12		; get half angle
	
; normalize half angle
dp3 r11.x, r2.xyz, r2.xyz		
rsq r11.xyz, r11.x			
mul r2.xyz, r2.xyz, r11.xyz	

; transform the half angle vector into texture space
dp3 r8.x,r3,r2
dp3 r8.y,r4,r2
dp3 r8.z,r5,r2

; half vector -> oT2 ps.1.4
mad oT2.xyz, r8, c33, c33 ; multiply by a half to bias, then add half 				

;transform the light vector with U, V, W
dp3 r8.x, r3, -c12
dp3 r8.y, r4, -c12
dp3 r8.z, r5, -c12

; light -> oD0
mad oD0.xyz, r8.xyz, c33, c33 ; multiply a half to bias, then add half
	  	  
mov oT0.xy, v7.xy


